<?php

/**
 * 
#GoDaddy Fix Rewrite
Options -Multiviews
Options +FollowSymLinks
 * 
 */

class IndexController extends Service_Controller_Action 
{
    
    public function init() 
    {
    	parent::init();
    }

    public function indexAction()
    {
		$oServiceUser = new Service_User();
		
		// Check user view permissions.
		$mUserId = null;
		if ($oServiceUser->getSessionUser()->privilege != Service_User::PRIVILEGE_ADMIN) 
		{
			$mUserId = (Service_User::USER_SEES_ONLY_OWN_FILES == $this->_aSettings['user_file_view']) ? $oServiceUser->getSessionUser()->user_id : null;
		}
		
		$oServiceFile = new Service_File();
		$oFiles = $oServiceFile->getFilesFromDirectory($this->_getParam('key', 0), $mUserId);
		$oFiles->setItemCountPerPage($this->_aSettings['pagination']);
		$oFiles->setCurrentPageNumber($this->_getParam('page', 1));
		
		$oServiceFolder = new Service_Folder();
		$oFolders = $oServiceFolder->getFolders($this->_getParam('key', 0), $mUserId);
		$oFolders->setItemCountPerPage($this->_aSettings['pagination']);
		$oFolders->setCurrentPageNumber($this->_getParam('page', 1));
		
		$aBreadcrump = $oServiceFolder->getBreadcrump($this->_getParam('key', 0));
		
		// Get total file size.
		$sTotalSize = $oServiceFile->getFileSizeSumToUser($oServiceUser->getSessionUser()->user_id);
		if (isset($this->_aSettings['user_storage_size']) && $this->_aSettings['user_storage_size'] != 0)
		{
			// Calculate for > 0 MB User Storage.
			$iPercentage = round((100/$this->_aSettings['user_storage_size']) *  $sTotalSize, 2);
		}
		else
		{
			// Calculate for 0 MB User Storage.
			$iDiskFreeSpace = disk_free_space(APPLICATION_UPLOADS_DIR . '/');
			$sDiskFreeSpace = $oServiceFile->getFileSize($iDiskFreeSpace);
			$this->view->sDiskFreeSpace = $sDiskFreeSpace;
		
			$sTotalSize = $oServiceFile->getFileSizeSum(true, $this->_getParam('key', 0));
			$iPercentage = round((100/$iDiskFreeSpace) *  $oServiceFile->getFileSizeSum(), 2);
		}
		
		// Search form
		$oSearchForm = new Application_Form_Search();
		
		if ($this->getRequest()->isPost()) {
            $aSearchFormData = $this->getRequest()->getPost();
            if (isset($aSearchFormData['formName']) && $aSearchFormData['formName'] == "searchForm" && $oSearchForm->isValid($aSearchFormData)) {
            	$this->_redirect('/index/search/key/' . $aSearchFormData['search']);
			}
		}
		
		$this->view->oFiles = $oFiles;
		$this->view->oFolders = $oFolders;
		$this->view->aBreadcrump = $aBreadcrump;
		$this->view->sTotalSize = $sTotalSize;
		$this->view->oSearchForm = $oSearchForm;
		$this->view->sKey = $this->_getParam('key', '');
		$this->view->iPercentage = $iPercentage;
		
		$this->folderAction();
		
		if ($iPercentage < 100 || $oServiceUser->getSessionUser()->privilege == Service_User::PRIVILEGE_ADMIN)
		{
			$this->uploadAction();
		}
    }
	
	public function deleteAction()
	{
	    // disable layout and view
	    $this->view->layout()->disableLayout();
	    $this->_helper->viewRenderer->setNoRender(true);
		
		// Check user permissions.
		$oSessionUser = Service_User::getSessionUser();
		if ($oSessionUser->delete != Service_User::FUNCTION_ALLOWED)
		{
			$this->_helper->flashMessenger->setNamespace('error')->addMessage("That's strictly forbidden!.");
			$this->_redirect('/');
		}
		
	    $sFileKey = $this->_getParam('key');
		
		if (!$sFileKey)
		{
			$this->_redirect('/');
		}
		
		// Delete link.
		$oServiceLink = new Service_Link();
		$oServiceLink->removeLink($sFileKey);
		
		$oServiceFile = new Service_File();
		$oFile = $oServiceFile->getFile($oServiceFile->getIdToKey($this->_getParam('key'), new Application_Model_Files()));
		
		$this->_checkPermission($oFile->user_id);
		
		// save the folder fo redirect.
		$iFolderId = $oFile->folder;
		
		$bootstrap = Zend_Controller_Front::getInstance()->getParam('bootstrap');
    	$aOptions = $bootstrap->getOption('monsterFile');
		
		//unlink($aOptions['directory'] . APPLICATION_UPLOADS_DIR . '/' . $oFile->file_name);
		//unlink($_SERVER['DOCUMENT_ROOT'] . $aOptions['directory'] . '/files/' . $oFile->file_name);
		unlink(APPLICATION_UPLOADS_DIR . '/' . $oFile->file_name);
		$oFile->delete();
		
		$oServiceFolder = new Service_Folder();
		$oFolder = $oServiceFolder->getFolder($iFolderId);
		
		$this->_helper->flashMessenger->setNamespace('success')->addMessage('The file deleted successfully.');

		if (!is_null($oFolder)) 
		{
			$this->_redirect('/folder/' . $oFolder->folder_key);
		}
		else
		{
			$this->_redirect('/');
		}
	}
	
	public function deletefolderAction()
	{
    	// disable layout and view
	    $this->view->layout()->disableLayout();
	    $this->_helper->viewRenderer->setNoRender(true);
		
		// Check user permissions.
		$oSessionUser = Service_User::getSessionUser();
		if ($oSessionUser->folder != Service_User::FUNCTION_ALLOWED)
		{
			$this->_helper->flashMessenger->setNamespace('error')->addMessage("That's strictly forbidden!.");
			$this->_redirect('/');
		}
		
		if (!$this->_hasParam('key'))
		{
			$this->_redirect('/');
		}
		
		$oServiceFolder = new Service_Folder();
		$oChildrenFolders = $oServiceFolder->getFolders($this->_getParam('key'));
		
		$oServiceFile = new Service_File();
		$oFiles = $oServiceFile->getFilesFromDirectory($this->_getParam('key'));
		
		if (count($oChildrenFolders) > 0 || count($oFiles) > 0)
		{
			$this->_helper->flashMessenger->setNamespace('info')->addMessage('Cannot Delete folder: The directory is not empty.');
			$this->_redirect('/');
		}
		
		// Delete folder.
		$oFolder = $oServiceFolder->getFolder($oServiceFolder->getIdToKey($this->_getParam('key'), new Application_Model_Folder()));
		$oFolder->delete();
		
		$this->_helper->flashMessenger->setNamespace('success')->addMessage('The folder deleted successfully.');
		$this->_redirect('/');
	}
	
	public function downloadAction()
	{
	    $sFileKey = $this->_getParam('key');
		
		if (!$sFileKey)
		{
			$this->_redirect('/');
		}
	    
		$oServiceFile = new Service_File();
		$oFile = $oServiceFile->getFile($oServiceFile->getIdToKey($this->_getParam('key'), new Application_Model_Files()));
		
		$this->_checkPermission($oFile->user_id);
		
	    header('Content-Type: ' . $oFile->mime_type);
	    header('Content-Disposition: attachment; filename="'. $oFile->name . '.' . $oFile->extension . '"');
		
		$bootstrap = Zend_Controller_Front::getInstance()->getParam('bootstrap');
    	$aOptions = $bootstrap->getOption('monsterFile');
		
	    //readfile($_SERVER['DOCUMENT_ROOT']. $aOptions['directory'] . '/files/' . $oFile->file_name);
	    readfile(APPLICATION_UPLOADS_DIR . '/' . $oFile->file_name);
	    //readfile('http://d.5me.net/files/' . $oFile->file_name);

		// increase download counter
		$oServiceFile->increaseDownloadCounter($oFile->file_id);
 
	    // disable layout and view
	    $this->view->layout()->disableLayout();
	    $this->_helper->viewRenderer->setNoRender(true);
	}
	
	public function fileAction() 
	{
		if (!$this->_hasParam('key'))
		{
			$this->_redirect('/');
		}
		
		$oServiceFile = new Service_File();
		$oFile = $oServiceFile->getFile($oServiceFile->getIdToKey($this->_getParam('key'), new Application_Model_Files()));
		
		$this->_checkPermission($oFile->user_id);
		
		// Calculate size.
		$sFileSize = $oServiceFile->getFileSize($oFile->size);
		
		$oServiceFolder = new Service_Folder();
		$oFolder = $oServiceFolder->getFolder($oFile->folder);
		
		$sFolderKey = !is_null($oFolder) ? $oFolder->folder_key : 0;
		$aBreadcrump = $oServiceFolder->getBreadcrump($sFolderKey);
		
		// Check link state.
		$oServiceLink = new Service_Link();
		$bIsFileLinked = $oServiceLink->isFileLinked($oFile->file_id);
		
		// load gravatar url.
		$oServiceUser = new Service_User();
		$oUser = $oServiceUser->getUser($oFile->user_id);
		$sMail = isset($oUser->email) ? $oUser->email : 0;
		$sGravatarUrl = $oServiceUser->getGravatarHash($sMail);
		
		$this->view->oFile = $oFile;
		$this->view->aBreadcrump = $aBreadcrump;
		$this->view->aBreadcrump2 = $aBreadcrump;
		$this->view->sGravatarUrl = $sGravatarUrl;
		$this->view->sFileSize = $sFileSize;
		$this->view->bLinkState = $bIsFileLinked;
	}

	public function searchAction()
	{
		$oServiceFile = new Service_File();
		$oResult = $oServiceFile->searchFiles($this->_getParam('key'));
		
		$this->view->oFiles = $oResult;
	}
	
	public function searchajaxAction()
	{
    	// disable layout and view
	    $this->view->layout()->disableLayout();
	    $this->_helper->viewRenderer->setNoRender(true);
		
		//var_dump($this->_getAllParams()); die;
		
		$oServiceFile = new Service_File();
		$oResult = $oServiceFile->searchFiles($this->_getParam('q'));
		
		$aResult = array();
		
		if ($oResult)
		{
			foreach ($oResult as $aItem)
			{
				//$aResult[] = $aItem['name'] . '.' . $aItem['extension'];
				$aResult[] = $aItem['name'];
			}
		}
		
		echo $this->_helper->json($aResult);
	}

	private function folderAction()
	{
		$oFormFolder = new Application_Form_Folder();
		$request = $this->getRequest();
		
		if (!$request->isPost()) 
	    {
	        $this->view->oFormFolder = $oFormFolder;
	        return;
	    }
		
		$aPost = $request->getPost();
		if (isset($aPost['submit']) && $aPost['submit'] == 'Upload') 
		{
			return;
		}
		
	    if (!$oFormFolder->isValid($request->getPost())) 
	    {
	        $this->view->oFormFolder = $oFormFolder;
	        return;
	    }
		
		$sKey = $this->_getParam('key', 0);
		
		$aFormData = $request->getPost();
		$oServiceFolder = new Service_Folder();
		$oServiceFolder->createFolder($aFormData['name'], $sKey);
		
		$this->_helper->flashMessenger->setNamespace('success')->addMessage('Folder created successfully.');
		if ($this->_hasParam('key')) 
		{
			$this->_redirect('/folder/' . $this->_getParam('key'));
		}
		else 
		{
			$this->_redirect('/');
		}
		
		
		$this->view->oFormFolder = $oFormFolder;
	}
	
	private function uploadAction() 
	{
    	// disable layout and view
	    //$this->view->layout()->disableLayout();
	    //$this->_helper->viewRenderer->setNoRender(true);
	    
		$bootstrap = Zend_Controller_Front::getInstance()->getParam('bootstrap');
    	$aOptions = $bootstrap->getOption('monsterFile');
		
		$form = new Application_Form_Upload();
		//$view = Zend_Layout::getMvcInstance()->getView();
		//$form->setAction($view->baseUrl('/index/upload/key/') . $this->_getParam('key'));
	    $form->setAction($this->view->url());
	
	    $request = $this->getRequest();
	
	    if (!$request->isPost()) 
	    {
	        $this->view->form = $form;
	        return;
	    }
		
		$aPost = $request->getPost();
		if (isset($aPost['submit']) && $aPost['submit'] == 'Create') 
		{
			return;
		}
	
	    if (!$form->isValid($request->getPost())) 
	    {
	        $this->view->form = $form;
	        return;
	    }
		
		// Check user permissions.
		$oSessionUser = Service_User::getSessionUser();
		if ($oSessionUser->upload != Service_User::FUNCTION_ALLOWED)
		{
			$this->_helper->flashMessenger->setNamespace('error')->addMessage("That's strictly forbidden!.");
			$this->_redirect('/');
		}
		
		$aOriginalFilename = pathinfo($form->file->getFileName());
    	$sNewFilename = 'file-' . uniqid() . '.' . $aOriginalFilename['extension'];
    	$form->file->addFilter('Rename', $sNewFilename);
	
	    try {
	        $form->file->receive();
			//chmod($_SERVER['DOCUMENT_ROOT']. $aOptions['directory'] . '/files/' . $sNewFilename, 0644);
			chmod(APPLICATION_UPLOADS_DIR . '/' . $sNewFilename, 0644);
			
	        //upload complete!
	        $location = $form->file->getFileName();
			$oServiceFolder = new Service_Folder();
			
			if ($this->_hasParam('key'))
			{
				$iFolder = $oServiceFolder->getIdToKey($this->_getParam('key', 0), new Application_Model_Folder());
			}
			else 
			{
				$iFolder = 0;
			}
			
			$iFolder = (int)$iFolder;
			
			$oServiceUser = new Service_User();
			
			// Calculate size.
			//var_dump(filesize($_SERVER['DOCUMENT_ROOT']. $aOptions['directory'] . '/files/' . $sNewFilename));
			$iSize = filesize(APPLICATION_UPLOADS_DIR . '/' . $sNewFilename);
			
			$aData = array(
				'name' => $aOriginalFilename['filename'],
				'file_name' => $sNewFilename,
				'description' => $form->getValue('description'),
				'folder' => $iFolder,
				'user_id' => $oServiceUser->getSessionUser()->user_id,
				'extension' => $aOriginalFilename['extension'],
				'mime_type' => $form->file->getMimeType(),
				'size' => $iSize
			);
			
			$oServiceFile = new Service_File();
			$oServiceFile->newFile($aData);
			
			//$this->_helper->flashMessenger->setNamespace('success')->addMessage('File uploaded successfully.');
			/*
			if ($this->_hasParam('key'))
			{
				//$this->_redirect('/index/index/key/' . $this->_getParam('key'));
			}
			else 
			{
				//$this->_redirect('/');
			}
			 */
			
	    } catch (Exception $exception) {
	        //error uploading file
	        $this->view->form = $form;
	    }
	}

	private function _checkPermission($iUserId)
	{
		// check permissions
		$oServiceUser = new Service_User();
		if ($oServiceUser->getSessionUser()->privilege != Service_User::PRIVILEGE_ADMIN
			&& $this->_aSettings['user_file_view'] == Service_User::USER_SEES_ONLY_OWN_FILES)  
		{
			if ($iUserId != $oServiceUser->getSessionUser()->user_id)
			{
				// File not owned by user.
				$this->_redirect('/');
			}
		}
	}
}