<?php

class Service_Folder extends Service_Structure
{
	protected $_oModelFolder;
	
	public function __construct() 
	{
		$this->_oModelFolder = new Application_Model_Folder();
	}
	
	/**
	 * Create a new folder.
	 * 
	 * @param String $sName
	 * @param int $iParentFolder = 0
	 * @return int PrimaryKey
	 */
	public function createFolder($sName, $mParentFolder = null)
	{
		$sKey = $this->createUniqueKey($this->_oModelFolder);
		$oServiceUser = new Service_User();
		
		$mParentFolder = $this->getIdToKey($mParentFolder, $this->_oModelFolder);
		
		$aData = array(
			$this->_oModelFolder->getForeignKey() => $sKey,
			'name' => $sName,
			'user_id' => $oServiceUser->getSessionUser()->user_id,
			'parent_folder_id' => (is_null($mParentFolder)) ? 0 : $mParentFolder,
		);
		
		return $this->_oModelFolder->insert($aData);
	}
	
	/**
	 * Count all folders.
	 * 
	 * @return int count
	 */
	public function countRows()
	{
	    return (int) $this->_oModelFolder->getAdapter()->fetchOne(
	        $this->_oModelFolder->select()->from($this->_oModelFolder, 'COUNT(folder_id)')
	    );
	}
	
	public function getBreadcrump($mActualFolderKey)
	{
		$aBreadcrump = array();
		
		if ($mActualFolderKey)
		{
			$iPrimaryId = $this->getIdToKey($mActualFolderKey, $this->_oModelFolder);
			
			do 
			{
				$oFolder = $this->getFolder($iPrimaryId);
				//array_unshift($aBreadcrump, array($oFolder->folder_key => $oFolder->name));
				$aBreadcrump[$oFolder->folder_key] = $oFolder->name;
				$iPrimaryId = $oFolder->parent_folder_id;
			}
			while ($oFolder->parent_folder_id > 0);
		}
		
		$aBreadcrump = array_reverse($aBreadcrump);
		
		return $aBreadcrump;
	}
	
	/**
	 * Returns one Folder.
	 * 
	 * @param int $iFolderId
	 * @return	Zend_Db_Table_Row_Abstract|null  
	 */
	public function getFolder($iFolderId)
	{
		return $this->_oModelFolder->fetchRow("folder_id = '" . $iFolderId . "'");
	}
	
	/**
	 * Get folders from directory.
	 * 
	 * @param mixed $mParentFolder
	 * @param mixed $mUserId
	 * @return Zend_Db_Table_Rowset_Abstract
	 */
	public function getFolders($mParentFolderKey = 0, $mUserId = null)
	{
		if ($mParentFolderKey)
		{
			$iPrimaryId = $this->getIdToKey($mParentFolderKey, $this->_oModelFolder);
		} 
		else
		{
			$iPrimaryId = 0;
		}
		
		$aWhere = array();
		$aWhere[] = "parent_folder_id = '" . $iPrimaryId . "'";
		
		if (!is_null($mUserId))
		{
			$aWhere[] = "user_id = '" . $mUserId . "'";
		}
		
		return Zend_Paginator::factory($this->_oModelFolder->fetchAll($aWhere, 'name ASC'));

		return $this->_oModelFolder->fetchAll($aWhere, 'name ASC');
	}

	/**
	 * Updates the folder fields.
	 * 
	 * @param int $iFolderId
	 * @param array $aData
	 * @return int affected rows
	 */	
	public function updateFolder($iFolderId, $aData) 
	{
		unset($aData['submit']);
		return $this->_oModelFolder->update($aData, "folder_id = '" . $iFolderId . "'");
	}
}
