<?php

class Service_Link
{
	protected $_oModelLinks;

	public function __construct() 
	{
		$this->_oModelLinks = new Application_Model_Links();
	}
	
	/**
	 * Create a link to a file.
	 * 
	 * @param String $sFileKey
	 * @return String $oLink->link_key
	 */
	public function createLink($sFileKey)
	{
		$oServiceFile = new Service_File();
		$oFile = $oServiceFile->getFile($oServiceFile->getIdToKey($sFileKey, new Application_Model_Files()));
		
		$sHash = $this->_generateHash($oFile->file_id);
		$oLink = $this->_oModelLinks->fetchRow("link_key = '" . $sHash . "'");

		if (is_null($oLink))
		{
			$aData = array(
				'link_key' => $sHash,
				'file_id' => $oFile->file_id,
				'user_id' => Service_User::getSessionUser()->user_id,
			);
			
			$iLinkId = $this->_oModelLinks->insert($aData);
			$oLink = $this->_oModelLinks->fetchRow("link_id = '" . $iLinkId . "'");
		}

		return $oLink->link_key;
	}
	
	/**
	 * Delivers a file from a link key.
	 * 
	 * @param String $sKey
	 * @return Zend_Db_Table_Row_Abstract|null
	 */
	public function getLink($sLinkKey)
	{
		$oEntry = $this->_oModelLinks->fetchRow("link_key = '" . $sLinkKey . "'");

		if (!$oEntry)
		{
			return null;
		}

		$oServiceFile = new Service_File();
		return $oServiceFile->getFile($oEntry->file_id);
	}

	/**
	 * Get the actual link state.
	 * 
	 * @param int $iFileId
	 * @return boolean
	 */
	public function isFileLinked($iFileId)
	{
		$oEntry = $this->_oModelLinks->fetchRow("file_id = '" . $iFileId . "'");
		return is_null($oEntry) ? false : true;
	}
	
	/**
	 * Delete existing link.
	 * 
	 * @param String $sFileKey
	 * @return int The number of rows deleted.
	 */
	public function removeLink($sFileKey)
	{
		$oServiceFile = new Service_File();
		$iFileId = $oServiceFile->getIdToKey($sFileKey, new Application_Model_Files());
		
		$oLink = $this->_oModelLinks->fetchRow("link_key = '" . $this->_generateHash($iFileId) . "'");
		
		if (isset($oLink))
		{
			return $oLink->delete();
		}
		return 0;
	}
	
	/**
	 * Generate a unique hash.
	 * 
	 * @param int $iFileId
	 * @return String Hash
	 */
	protected function _generateHash($iFileId)
	{
		$oServiceFile = new Service_File();
		$oFile = $oServiceFile->getFile($iFileId);
		
		return md5($oFile->file_name . $oFile->date_input);
	}
}