<?php

class Service_User {
	const PRIVILEGE_ADMIN = 'admin';
	const PRIVILEGE_USER = 'user';
	
	const USER_STATE_ACTIVE = 1;
	const USER_STATE_INACTIVE = 0;
	
	const FUNCTION_ALLOWED = 1;
	const FUNCTION_NOT_ALLOWED = 0;
	
	const USER_SEES_ALL_FILES = 0;
	const USER_SEES_ONLY_OWN_FILES = 1;

	protected $_oModelUser;
	
	public function __construct() {
		$this->_oModelUser = new Application_Model_User();
	}
	
	/**
	 * Delete the User.
	 * 
	 * @param int $iUserId
	 * @return int The number of rows deleted.
	 */
	public function deleteUser($iUserId) {
		$oUser = $this->getUser($iUserId);
		return $oUser->delete();
	}
	
	public function getAll() {
		return $this->_oModelUser->fetchAll();
	}
	
	static public function getFullName($iUserId = 0) {
		/*
		 * Muss gemacht werden, weil ich im View
		 * so darauf zugreife: Service_User::getFullName(1);
		 */
		$oModelUser = new Application_Model_User();
		$oUser = $oModelUser->fetchRow($oModelUser->select()->where('user_id = ?', $iUserId));
		
		if (is_null($oUser)) {
			return "Unknown";
		} else {
			return $oUser->firstName . ' ' . $oUser->lastName;
		}
	}
	
	/**
	 * Get the Gravatar url.
	 * 
	 * @param String $sMail
	 * @return String Hash
	 */
	static public function getGravatarHash($sMail = 0) {
		$sUrl = 'http://www.gravatar.com/avatar/';
		$sHash = md5(strtolower(trim($sMail)));
		return $sUrl . $sHash;
	}
	
	/**
	 * Lädt den Benutzer aus der Session.
	 * @return object
	 */
	static public function getSessionUser() {
		$oStorage = new Zend_Auth_Storage_Session();
		return $oStorage->read();
	}
	
	/**
	 * Delivers the User-Object.
	 * 
	 * @param int $iUserId
	 * @return Zend_Db_Table_Row_Abstract|null The row results per the
     *     Zend_Db_Adapter fetch mode, or null if no row found.
	 */
	public function getUser($iUserId) {
		$oModelUser = new Application_Model_User();
		return $oModelUser->fetchRow($oModelUser->select()->where('user_id = ?', $iUserId));
	}
	
	/**
	 * Delivers the User-Object.
	 * 
	 * @param String $sMail
	 * @return Zend_Db_Table_Row_Abstract|null The row results per the
     *     Zend_Db_Adapter fetch mode, or null if no row found.
	 */
	public function getUserByMail($sMail) {
		$oModelUser = new Application_Model_User();
		return $oModelUser->fetchRow($oModelUser->select()->where('email = ?', $sMail));
	}
	
	/**
	 * Erstellt einen neuen Benutzer.
	 * 
	 * @param array $aData
	 * @param boolean $bUserStateActive = true
	 * @return int $iUserId
	 */
	public function newUser($aData, $bUserStateActive = true) {
		$aData['active'] = (true === $bUserStateActive) ? self::USER_STATE_ACTIVE : self::USER_STATE_INACTIVE;
		$aData['password'] = md5($aData['password']);
		
		if (!isset($aData['privilege']))
		{
			$aData['privilege'] = self::PRIVILEGE_USER;
		}
		
		unset($aData['csrf']);
		unset($aData['submit']);
		unset($aData['save']);
		return $this->_oModelUser->insert($aData);
	}
	
	/**
	 * Update user in table and session.
	 * 
	 * @param array $aData
	 * @param int $iUserId
	 * @param boolean $bUpdateSession = true
	 */
	public function updateUser($aData, $iUserId, $bUpdateSession = true)
	{
		$oUser = $this->getUser($iUserId);
		
		if (isset($aData['password']) && strlen($aData['password']) > 0)
		{
			$aData['password'] = md5($aData['password']);
		}
		else
		{
			unset($aData['password']);
		}
		unset($aData['save']);
		
		$this->_oModelUser->update($aData, "user_id = '" . $iUserId . "'");
		
		if (true === $bUpdateSession)
		{
			// fetch data in auth storage
			$authStorage = Zend_Auth::getInstance()->getStorage();
			$authData = $authStorage->read();
			// change values in $authData
			$authStorage->write($this->getUser($iUserId));
		}
	}
	
	public function createPassword($length) {
		$chars = "234567890abcdefghijkmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
		$i = 0;
		$password = "";
		while ($i <= $length) {
			$password .= $chars{mt_rand(0,strlen($chars))};
			$i++;
		}
		
		return $password;
	}
}